#define F_CPU 16000000

#include <avr/io.h>
#include <util/delay.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "atmega_bmp180_lib.h"
#include "dht.h"
#include "gfx.h"
#include "lcd.h"
#include "twi_lib.h"

//print a message to LCD
void print_mess(int16_t x, int16_t y, uint16_t color, const char *mess)
{
	//GFX_set_rotation(0);
	GFX_set_cursor(x, y);
	GFX_set_text_color(color);
	GFX_print_str(mess);
}

//compute the heatindex
float heatindex(float T,float RH){
	float hi = -42.379 + 2.04901523*T + 10.14333127*RH - .22475541*T*RH - .00683783*T*T ;
	hi -= .05481717*RH*RH + .00122874*T*T*RH + .00085282*T*RH*RH - .00000199*T*T*RH*RH;
	return hi;
}

// celsius to fahrenheit
float celtofar(float temp){
	return temp*9/5 + 32;
}

//fahrenheit to celsius
float fartocel(float temp){
	return temp*5/9 -32;
}

int main(void){
	char t_mess[30];
	char p_mess[30];
	char h_mess[30];
	char hi_mess[30];
	char a_mess[30];

	int32_t temperature = 0;
	int32_t altitude = 0;
	int32_t pressure = 0;
	int16_t BMP180_calibration_int16_t[8];
	int16_t BMP180_calibration_uint16_t[3];
	int16_t sizeoffont = 16;
	uint8_t err = 0;

	float temperature2 = 0;
	float auxtemp = 0;
	float hi;
	float humidity = 0;

	LCD_init();
	GFX_init(128,128);

	i2cSetBitrate(200);

	//prepare bmp180

	BMP180_Calibration(BMP180_calibration_int16_t,BMP180_calibration_uint16_t,&err);

	LCD_fill_screen(ST7735_BLACK);

	while(1){
		//get temperature and pressure	
		bmp180Convert(BMP180_calibration_int16_t,BMP180_calibration_uint16_t,&temperature,&pressure,&err);
		
		//compute altitude
		altitude = bmp180CalcAltitude(pressure);
		
		//get temperature and humidity
		dht_gettemperaturehumidity(&temperature2, &humidity);

		//compute the average of the 2 temperature values
		auxtemp = (temperature/10 + (temperature%10)/10. + temperature2)/2;
		memset(t_mess,0,sizeof(t_mess));
		sprintf(t_mess,"Temp: %.1fC", auxtemp);
		
		memset(h_mess,0,sizeof(h_mess));
		sprintf(h_mess,"Humi: %.1f%%",humidity);
		
		memset(p_mess,0,sizeof(p_mess));
		sprintf(p_mess,"Pres: %ldkPa",pressure/1000);
		
		memset(a_mess,0,sizeof(a_mess));
		sprintf(a_mess,"Alti: %ld",altitude);
		
		//compute the heat index
		hi = heatindex(celtofar(auxtemp),humidity);
		memset(hi_mess,0,sizeof(hi_mess));
		sprintf(hi_mess,"Feels: %.1fC",fartocel(hi));
		
		//print messages
		print_mess(0, 32 , ST7735_WHITE, t_mess);
		print_mess(0, 32 + sizeoffont , ST7735_WHITE, h_mess);
		print_mess(0, 32 + 2 * sizeoffont, ST7735_WHITE, p_mess);
		print_mess(0, 32 + 3 *sizeoffont, ST7735_WHITE, hi_mess);
		print_mess(0, 32 + 4 * sizeoffont, ST7735_WHITE, a_mess);
		_delay_ms(2000);
	}


	return 0;

}